@@ -18,16 +18,12 @@
 
 #include <algorithm>
 
-#include <common/regex.hpp>
-
-#include <cupt/file.hpp>
 #include <cupt/config.hpp>
 #include <cupt/cache.hpp>
 #include <cupt/cache/binarypackage.hpp>
 #include <cupt/cache/binaryversion.hpp>
 #include <cupt/system/state.hpp>
 
-#include <internal/filesystem.hpp>
 #include <internal/cacheimpl.hpp>
 #include <internal/regex.hpp>
 #include <internal/cachefiles.hpp>
@@ -297,217 +293,7 @@ pair< string, string > Cache::getLocalizedDescriptions(const shared_ptr< const B
 // static
 bool Cache::verifySignature(const shared_ptr< const Config >& config, const string& path)
 {
-	auto debugging = config->getBool("debug::gpgv");
-	if (debugging)
-	{
-		debug("verifying file '%s'", path.c_str());
-	}
-
-	auto keyringPath = config->getString("gpgv::trustedkeyring");
-	if (debugging)
-	{
-		debug("keyring file is '%s'", keyringPath.c_str());
-	}
-
-	auto signaturePath = path + ".gpg";
-	if (debugging)
-	{
-		debug("signature file is '%s'", signaturePath.c_str());
-	}
-
-	if (!internal::fs::fileExists(signaturePath))
-	{
-		if (debugging)
-		{
-			debug("signature file '%s' doesn't exist", signaturePath.c_str());
-		}
-		return 0;
-	}
-
-	// file checks
-	{
-		string openError;
-		File file(signaturePath, "r", openError);
-		if (!openError.empty())
-		{
-			if (debugging)
-			{
-				debug("unable to read signature file '%s': %s",
-						signaturePath.c_str(), openError.c_str());
-			}
-			return false;
-		}
-	}
-	{
-		string openError;
-		File file(keyringPath, "r", openError);
-		if (!openError.empty())
-		{
-			if (debugging)
-			{
-				debug("unable to read keyring file '%s': %s",
-						keyringPath.c_str(), openError.c_str());
-			}
-			return false;
-		}
-	}
-
-	bool verifyResult = false;
-	try
-	{
-		string gpgCommand = string("gpgv --status-fd 1 --keyring ") + keyringPath +
-				' ' + signaturePath + ' ' + path + " 2>/dev/null || true";
-		string openError;
-		File gpgPipe(gpgCommand, "pr", openError);
-		if (!openError.empty())
-		{
-			fatal("unable to open gpg pipe: %s", openError.c_str());
-		}
-
-		smatch m;
-		auto gpgGetLine = [&gpgPipe, &m, &debugging]() -> string
-		{
-			static const sregex sigIdRegex = sregex::compile("\\[GNUPG:\\] SIG_ID");
-			static const sregex generalRegex = sregex::compile("\\[GNUPG:\\] ");
-			string result;
-			do
-			{
-				gpgPipe.getLine(result);
-				if (debugging && !gpgPipe.eof())
-				{
-					debug("fetched '%s' from gpg pipe", result.c_str());
-				}
-			} while (!gpgPipe.eof() && (
-						regex_search(result, m, sigIdRegex, regex_constants::match_continuous) ||
-						!regex_search(result, m, generalRegex, regex_constants::match_continuous)));
-
-			if (gpgPipe.eof())
-			{
-				return "";
-			}
-			else
-			{
-				return regex_replace(result, generalRegex, "");
-			}
-		};
-
-
-		auto status = gpgGetLine();
-		if (status.empty())
-		{
-			// no info from gpg at all
-			fatal("gpg: '%s': no info received", path.c_str());
-		}
-
-		// first line ought to be validness indicator
-		static const sregex messageRegex = sregex::compile("(\\w+) (.*)");
-		if (!regex_match(status, m, messageRegex))
-		{
-			fatal("gpg: '%s': invalid status string '%s'", path.c_str(), status.c_str());
-		}
-
-		string messageType = m[1];
-		string message = m[2];
-
-		if (messageType == "GOODSIG")
-		{
-			string furtherInfo = gpgGetLine();
-			if (furtherInfo.empty())
-			{
-				fatal("gpg: '%s': error: unfinished status", path.c_str());
-			}
-
-			if (!regex_match(furtherInfo, m, messageRegex))
-			{
-				fatal("gpg: '%s': invalid further info string '%s'", path.c_str(), furtherInfo.c_str());
-			}
-
-			string furtherInfoType = m[1];
-			string furtherInfoMessage = m[2];
-			if (furtherInfoType == "VALIDSIG")
-			{
-				// no comments :)
-				verifyResult = 1;
-			}
-			else if (furtherInfoType == "EXPSIG")
-			{
-				warn("gpg: '%s': expired signature: %s", path.c_str(), furtherInfoMessage.c_str());
-			}
-			else if (furtherInfoType == "EXPKEYSIG")
-			{
-				warn("gpg: '%s': expired key: %s", path.c_str(), furtherInfoMessage.c_str());
-			}
-			else if (furtherInfoType == "REVKEYSIG")
-			{
-				warn("gpg: '%s': revoked key: %s", path.c_str(), furtherInfoMessage.c_str());
-			}
-			else
-			{
-				warn("gpg: '%s': unknown error: %s %s",
-						path.c_str(), furtherInfoType.c_str(), furtherInfoMessage.c_str());
-			}
-		}
-		else if (messageType == "BADSIG")
-		{
-			warn("gpg: '%s': bad signature: %s", path.c_str(), message.c_str());
-		}
-		else if (messageType == "ERRSIG")
-		{
-			// gpg was not able to verify signature
-
-			// maybe, public key was not found?
-			bool publicKeyWasNotFound = false;
-			auto detail = gpgGetLine();
-			if (!detail.empty())
-			{
-				if (!regex_match(detail, m, messageRegex))
-				{
-					fatal("gpg: '%s': invalid detailed info string '%s'", path.c_str(), detail.c_str());
-				}
-				string detailType = m[1];
-				string detailMessage = m[2];
-				if (detailType == "NO_PUBKEY")
-				{
-					publicKeyWasNotFound = true;
-
-					// the message looks like
-					//
-					// NO_PUBKEY D4F5CE00FA0E9B9D
-					//
-					warn("gpg: '%s': public key '%s' not found", path.c_str(), detailMessage.c_str());
-				}
-			}
-
-			if (!publicKeyWasNotFound)
-			{
-				warn("gpg: '%s': could not verify signature: %s", path.c_str(), message.c_str());
-			}
-		}
-		else if (messageType == "NODATA")
-		{
-			// no signature
-			warn("gpg: '%s': empty signature", path.c_str());
-		}
-		else if (messageType == "KEYEXPIRED")
-		{
-			warn("gpg: '%s': expired key: %s", path.c_str(), message.c_str());
-		}
-		else
-		{
-			warn("gpg: '%s': unknown message received: %s %s",
-					path.c_str(), messageType.c_str(), message.c_str());
-		}
-	}
-	catch (Exception&)
-	{
-		warn("error while verifying signature for file '%s'", path.c_str());
-	}
-
-	if (debugging)
-	{
-		debug("the verify result is %u", (unsigned int)verifyResult);
-	}
-	return verifyResult;
+	return internal::cachefiles::verifySignature(*config, path);
 }
 
 string Cache::getPathOfCopyright(const shared_ptr< const BinaryVersion >& version)
